package org.openxma.dsl.reference.dao;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.reference.dao.VODataDao;
import org.openxma.dsl.reference.model.VOData;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity VODataDao.
 * @see VODataDao
 */
public interface VODataDaoGen extends GenericDao<VOData,String>, EntityFactory<VOData> {
    /**
     * Factory method to create an empty <code>VOData</code> entity with an already assigned identifier.
     * The advantage of this approach, coupled with the fact that we're using the id for equals,
     * is that equality test's are immediately possible and therefore independent from the persistence state
     * of the entity. This in contrast to other solutions where auto generated Keys are used
     * which must wait until the object is persisted or differentiate between transient and managed states.
     * @return a new (transient), empty <code>VOData</code> entity with an already assigned identifier.
     */
    VOData create();
}
