package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dao.CategoryDao;
import org.openxma.dsl.reference.dao.CertificateDao;
import org.openxma.dsl.reference.dao.ProductDao;
import org.openxma.dsl.reference.dao.ProductImageDao;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.dto.ProductView;
import org.openxma.dsl.reference.model.Certificate;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.service.ProductService;
import org.openxma.dsl.reference.types.valueobject.ProductId;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class ProductServiceGenImpl implements ProductService {
    protected final Log logger = LogFactory.getLog(getClass());
    @Autowired(required=false)
    protected Mapper mapper;
    protected ProductDao productDao;
    protected SupplierDao supplierDao;
    protected ProductImageDao productImageDao;
    protected CategoryDao categoryDao;
    protected CertificateDao certificateDao;
    
    @Autowired
    public void setProductDao(ProductDao productDao) {
        this.productDao = productDao;
    }

    @Autowired
    public void setSupplierDao(SupplierDao supplierDao) {
        this.supplierDao = supplierDao;
    }

    @Autowired
    public void setProductImageDao(ProductImageDao productImageDao) {
        this.productImageDao = productImageDao;
    }

    @Autowired
    public void setCategoryDao(CategoryDao categoryDao) {
        this.categoryDao = categoryDao;
    }

    @Autowired
    public void setCertificateDao(CertificateDao certificateDao) {
        this.certificateDao = certificateDao;
    }
    
    @Transactional
	public ProductView save(ProductView productView) {
        Assert.notNull(productView, "parameter 'productView' must not be null");
        Product product = this.mapper.createAndMapOne(productView, Product.class,"saveProduct");
        Supplier supplier = this.mapper.createAndMapOne(productView, Supplier.class,"supplier");
        supplier = supplierDao.load(supplier.getOid());
        product.setSupplier(supplier);
        if (null != productView.getCert1()) {
            Certificate optionalCert1 = this.mapper.createAndMapOne(productView.getCert1(), Certificate.class,"cert1");
            optionalCert1 = certificateDao.load(optionalCert1.getOid());
            product.setCert1(optionalCert1);
        }
        if (null != productView.getCert2()) {
            Certificate optionalCert2 = this.mapper.createAndMapOne(productView.getCert2(), Certificate.class,"cert2");
            optionalCert2 = certificateDao.load(optionalCert2.getOid());
            product.setCert2(optionalCert2);
        }
        if (null != productView.getCert3()) {
            Certificate optionalCert3 = this.mapper.createAndMapOne(productView.getCert3(), Certificate.class,"cert3");
            optionalCert3 = certificateDao.load(optionalCert3.getOid());
            product.setCert3(optionalCert3);
        }
        this.productDao.saveOrUpdate(product);
        return this.mapper.mapOne(product,new ProductView());
    }
    
    @Transactional(readOnly = true)
	public ProductView loadProductView(ProductId oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.productDao.load(oid),ProductView.class);
    }
    
    @Transactional
	public void update(ProductView productView) {
        Assert.notNull(productView, "parameter 'productView' must not be null");
        Product mappedProduct = this.mapper.createAndMapOne(productView, Product.class);
        Product product = this.productDao.load(mappedProduct.getOid());
        if (!product.getVersion().equals(mappedProduct.getVersion())) {
            throw new OptimisticLockingFailureException("Entity 'Product' was updated or deleted by another transaction.");
        }
        this.mapper.mapOne(productView, product,"updateProduct");
    }
    
    @Transactional
	public void deleteProduct(ProductId oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        this.productDao.delete(oid);
    }
        
    @Transactional
    public ProductView saveOrUpdate(ProductView productView) {
        Assert.notNull(productView, "parameter 'productView' must not be null");
        if ( productView.getVersion() == null ) {
            return this.save(productView);
        }
        else {
            this.update(productView);
            return productView;
        }
    }

    
}
